#include	<stdio.h>
#include	<math.h>
#include	<stdlib.h>

#define		INPUT_FILE	"synspec.inp"
#define		OUTPUT_FILE	"synspec.dat"

/* Program to synthesize a spectrum given an input file (SYNSPEC.INP)
   that contains (1) the number of peaks, (2) position and intensity
   for each peak.

   User (interactive) input is (1) lineshape (0=gaussian, 1=lorentzian),
   (2) linewidth, (3) energy/wavelength range to calculate, (4) spacing
   of points to compute across the specified range (evenly spaced).

   Output is to file SYNSPEC.DAT and is a list of (x,y) points.

   Karl Irikura 5/10/95
   National Institute of Standards and Technology, Gaithersburg, MD 20899, USA
   Please cite or acknowledge this program if you use it to produce
   publications.
*/

float	width;

void main(void)
{
	int		npeaks, lineshape, i, j;
    float	*pos, *intens, emin, emax, deltae, x, y;
    char	line[200];
    FILE	*fp;
    float	gaussian(float, float), lorentzian(float, float);
    int		getfloat(float *);

    printf("This program synthesizes a spectrum from input peak positions\n");
    printf("and intensities (from file SYNSPEC.INP) and can use either\n");
    printf("a Gaussian or Lorentzian peak shape.  Output is to SYNSPEC.DAT.\n\n");

    if ((fp=fopen(INPUT_FILE,"r")) == NULL) {
    	printf("Failed opening file %s for input.\n", INPUT_FILE);
        exit(1);
    }
	fgets(line, 200, fp);
    if (sscanf(line, "%d", &npeaks) != 1) {
    	printf("First line of input must contain the number of lines.\n");
        exit(1);
    }
    /* allocate memory for line information */
	pos = (float *) malloc(npeaks*sizeof(float));
    intens = (float *) malloc(npeaks*sizeof(float));
    /* read NPTS (position, intensity) pairs from input.  Default intensity=1. */
    for (i=0; i<npeaks; i++) {
    	if (fgets(line,200,fp) == NULL) {
        	printf("Ran out of data before reading %d points!\n", npeaks);
            exit(1);
        }
        intens[i] = 1.;
        sscanf(line,"%f %f", pos+i, intens+i);
    }
    fclose(fp);
	printf("%d lines read.  Input file closed.\n\n", npeaks);
    /* read standard input */
    printf("Lineshape (0=gaussian, 1=lorentzian)? ");
	getfloat(&width);
	lineshape = (width == 1.0 ? 1 : 0);
    printf("Using %s lineshape.\n", (lineshape ? "Lorentzian" : "Gaussian"));
	printf("Linewidth (FWHM)? ");
	getfloat(&width);
	if (width < 1.e-30)
		width = 1.;
	printf("Using FWHM of %f (s = ", width);
	width = (lineshape ? width/2 : width/(2*sqrt(log(2.))));
	printf("%f).\n", width);
    printf("Spectral range to calculate:\n\tminimum? ");
    getfloat(&emin);
    printf("\tmaximum? ");
    getfloat(&emax);
    printf("Step size across the spectral range? ");
    getfloat(&deltae);
    if ((emax-emin)/deltae < 0.)
    	deltae = -deltae;
    /* calculate data and write to output */
    if ((fp=fopen(OUTPUT_FILE,"w")) == NULL) {
    	printf("Failed opening %s for output.\n", OUTPUT_FILE);
        exit(1);
    }
    for (x=emin; x<=emax; x+=deltae) {
        y = 0.;
        for (j=0; j<npeaks; j++)
			y += intens[j]*(lineshape? lorentzian(x, pos[j]) : gaussian(x, pos[j]));
       	fprintf(fp, " %f %f\n", x, y);
    }
    fclose(fp);
    /* free memory */
	free(pos);
    free(intens);
}

int getfloat(float *value)
/* read a line from standard input, then read a float from the line.
   Return same thing as sscanf() */
{
	char	line[200];

    gets(line);
    return(sscanf(line, "%f", value));
}

float gaussian(float x, float peak)
/* amplitude at X of gaussian peak centered at PEAK.  WIDTH is global. */
{
	float	t;

	t = (x-peak)/width;
    t = exp(-t*t);
    t /= width*sqrt(M_PI);
    return(t);
}

float lorentzian(float x, float peak)
/* amplitude at X of lorentzian peak centered at PEAK. WIDTH is global. */
{
	float	t;

    t = (x-peak)*(x-peak) + width*width;
    t = width/(M_PI * t);
    return(t);
}
